# noqa: D100
from atsphinx.og_article import __version__ as version

# -- Project information
project = "atsphinx-og-article"
copyright = "2023, Kazuya Takei"
author = "Kazuya Takei"
release = version

# -- General configuration
extensions = [
    "rst_pypi_ref.sphinx",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
]
templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for HTML output
html_theme = "furo"
html_title = f"{project} v{version}"
html_static_path = ["_static"]


# -- Options for extensions
# sphinx.ext.intersphinx
intersphinx_mapping = {
    "sphinx": ("https://www.sphinx-doc.org/en/master", None),
}
# sphinx.ext.todo
todo_include_todos = True


def setup(app):  # noqa: D103
    app.add_object_type(
        "confval",
        "confval",
        objname="configuration value",
        indextemplate="pair: %s; configuration value",
    )



"""Support article tpye's properties of Open Graph for Sphinx."""

from docutils import nodes
from sphinx.application import Sphinx

from . import models, processors

__version__ = "0.1.1"


def skip_node(self, node: models.og_article):
    """Minimum function to skip when builder visit it."""
    raise nodes.SkipNode()


def setup(app: Sphinx):  # noqa: D103
    app.add_config_value("og_article_timezone", None, "env", [str])
    app.add_directive("og-article", models.OgArticleDirective)
    app.add_node(
        models.og_article,
        html=(skip_node, None),
        latex=(skip_node, None),
        text=(skip_node, None),
        man=(skip_node, None),
        texinfo=(skip_node, None),
    )
    app.connect("html-page-context", processors.add_metatags)
    return {
        "version": __version__,
        "env_version": 1,
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



"""Object models in doctree domain."""

from datetime import datetime

from dateutil.parser import parse
from dateutil.tz import gettz
from docutils import nodes
from sphinx.config import Config
from sphinx.util.docutils import SphinxDirective


class og_article(nodes.General, nodes.Element):
    """Node class to manage article metadata."""

    pass


def complement_time_options(options: dict, config: Config) -> dict:
    """Convert datetime object flexibly.

    :returns: datetime object with tzinfo.
    """
    # Define vars
    published_time = options.get("published_time", None)
    modified_time = options.get("modified_time", None)
    # Append timezone if need
    if config.og_article_timezone:
        tzinfo = gettz(config.og_article_timezone)
        if published_time and not published_time.tzinfo:
            published_time = published_time.replace(tzinfo=tzinfo)
        if modified_time and not modified_time.tzinfo:
            modified_time = modified_time.replace(tzinfo=tzinfo)
    # Complete times
    if published_time is None and modified_time is not None:
        published_time = modified_time
    elif published_time is not None and modified_time is None:
        modified_time = published_time
    elif published_time is None and modified_time is None:
        now = datetime.now()
        if config.og_article_timezone:
            now = now.replace(tzinfo=gettz(config.og_article_timezone))
        published_time = now
        modified_time = now
    # Return
    options["published_time"] = published_time
    options["modified_time"] = modified_time
    return options


class OgArticleDirective(SphinxDirective):
    """Definition of directive to declare article metadata."""

    has_content = True

    option_spec = {
        "published_time": parse,
        "modified_time": parse,
    }

    def run(self):  # noqa: D102
        node = og_article()
        node.attributes = complement_time_options(self.options, self.env.config)
        return [
            node,
        ]



"""Context processrors for Sphinx."""

from typing import Optional
from xml.etree import ElementTree as ET

from sphinx import addnodes
from sphinx.application import Sphinx

from . import models


def add_metatags(
    app: Sphinx,
    pathname: str,
    templatename: str,
    context: dict,
    doctree: Optional[addnodes.document] = None,
):
    """Pick og attributes from document and inject into metatags."""
    if not doctree:
        return
    targets = list(doctree.findall(models.og_article))
    if not targets:
        return
    if "metatags" not in context:
        context["metatags"] = ""
    node: models.og_article = targets[0]
    # Append time properties
    metatags = [
        ET.Element(
            "meta",
            {
                "property": "article:published_time",
                "content": node["published_time"].isoformat(timespec="seconds"),
            },
        ),
        ET.Element(
            "meta",
            {
                "property": "article:modified_time",
                "content": node["modified_time"].isoformat(timespec="seconds"),
            },
        ),
    ]
    metatags = b"\n".join([ET.tostring(e) for e in metatags])
    context["metatags"] += f"\n{metatags}"
